/* 
Maps with SG
Example of how to create maps with the SGPLOT procedure.
Data is prepared using the GPROJECT procedure from SAS/Graph

Author: Frank Poppe
	PW Consulting, the Netherlands
	Frank.Poppe@PWconsulting.nl

March 2015

A map is drawn for The Nehtherlands, showing the provinces within.
In the area around the country the borders of neighbouring countries are drawn as well.
*/

/* Select the coordinates from the countries that might be needed for the surroundings */
proc sql ;
create view surroundings as
select * from mapsGFK.world
where ID in ( 'BE', 'LU' , 'DE',  'FR', 'GB' , 'DK' ) 
;

/* Make sure the data set does not exist */
proc datasets library=work ;
delete NL_surroundings ;
run ;
/* Create a new dataset with the coordinates for both the surrounding contries
and the provincies within the Netherlands */
append base = NL_surroundings data = mapsGFK.netherlands (drop = ID1 ) ;
run ;
append base = NL_surroundings data = surroundings ( drop = idname cont iso isoalpha2 ) ;
run ;
quit ;

/* Project these coordinates using the projection common for official maps of the Netherlands.
The internationally agreed code for that projection is "EPSG:28992".
GPROJECT knows about these codes and the projection parameters it stands for */
proc gproject
	data = NL_surroundings
	latlong
	to="EPSG:28992"
	out=NL_surroundings_projected
;
id id ;
run ;

/* Use soms macro variables for flexibility
Determine the 'bounding box' for the projected data, 
for only the coordinates for the Netherlands */
%let mapsSource = NL_surroundings_projected ;
%let whereClause = substr(ID,1,2) = 'NL'  ;
proc sql ;
create table frame as
	select 	min ( x ) as left 
		,	max ( x ) as right
		,	min ( y  ) as bottom
		,	max ( y  ) as top 
	from &mapsSource 
	where &whereClause 
;

/* Use the ADJUSTFRAME macro to create a margin around the 
bounding box. The parameter .3 creates a 30% margin 
(with respect to the largest original range in horizontal
or vertical direction) 
The new margins are returned as macro variables */
%adjustFrame ( frame , framemargin = .3 ) ;

%put &left ... &right ;
%put &bottom ... &top ;

/* The segments (the polygons which belong to the same geographical unit)
have to get unique numbers for Proc SGPLOT.
Also create a data set containing unique ID values.
Rename and select variables */
data 
	segments_renumbered ( keep =  id newseg  x y rename = ( newseg = segment) )
	units ( keep = id  ) 
;
	set NL_surroundings_projected ;
	by id notsorted segment ;
	retain 
		newseg 0 
	;
	if first.id then do ;
		output units ;
	end ;
	if first.segment then newseg+1 ;
	output segments_renumbered ;
run ;

/* Use the data set with unique id's to create a table 
that can be used as 'attribute table' in Proc SGPLOT.
Only two colors are used: within or without the Netherlands.
A measure on the geographic regions could be used as well,
and in a real world application probably will 
ID is a reserved variable for attribute maps, 
the VALUE var must contain the group value for which the attributes are set
*/
data attrs  ;
	set units ;
	length value $ 5 ;

	value = id ; 
	if id =: "NL" then fillcolor = 'salmon' ;
		else fillcolor = 'silver' ;
	id = 'A01' ;
run ;

ods _ALL_ close ;
ods graphics on / 
 reset
 height=800px width=800px
 border =off 
 imagename = 'NLaround' 
;
ods listing ;

/* Create the map */
proc sgplot 
	data = segments_renumbered
	dattrmap = attrs 
	aspect = 1
	noautolegend
	noborder
;

	xaxis
		grid
		values = ( &left &right )
		display=none
	;
	yaxis 
		grid
		values =  ( &bottom &top )
		display=none
	;
	polygon 
		x = x y = y 
		id = segment / 
		nooutline fill
		group = id 
		attrid = A01
		dataskin = sheen
	;

run ;

ODS _ALL_ close ;

